@@ -701,12 +701,12 @@ void emperor_add(struct uwsgi_emperor_scanner *ues, char *name, time_t born, cha
 
 	gettimeofday(&tv, NULL);
 	int now = tv.tv_sec;
-	uint64_t micros = (tv.tv_sec * 1000 * 1000) + tv.tv_usec;
+	uint64_t micros = (tv.tv_sec * 1000ULL * 1000ULL) + tv.tv_usec;
 
 	// blacklist check
 	struct uwsgi_emperor_blacklist_item *uebi = uwsgi_emperor_blacklist_check(name);
 	if (uebi) {
-		uint64_t i_micros = (uebi->last_attempt.tv_sec * 1000 * 1000) + uebi->last_attempt.tv_usec + uebi->throttle_level;
+		uint64_t i_micros = (uebi->last_attempt.tv_sec * 1000ULL * 1000ULL) + uebi->last_attempt.tv_usec + uebi->throttle_level;
 		if (i_micros > micros) {
 			return;
 		}
@@ -1746,7 +1746,8 @@ void emperor_send_stats(int fd) {
 		goto end0;
 	struct uwsgi_emperor_scanner *ues = emperor_scanners;
 	while (ues) {
-		uwsgi_stats_str(us, ues->arg);
+		if (uwsgi_stats_str(us, ues->arg))
+			goto end0;
 		ues = ues->next;
 		if (ues) {
 			if (uwsgi_stats_comma(us))
@@ -1889,7 +1890,7 @@ void emperor_send_stats(int fd) {
 	size_t remains = us->pos;
 	off_t pos = 0;
 	while (remains > 0) {
-		int ret = uwsgi_waitfd_write(client_fd, uwsgi.shared->options[UWSGI_OPTION_SOCKET_TIMEOUT]);
+		int ret = uwsgi_waitfd_write(client_fd, uwsgi.socket_timeout);
 		if (ret <= 0) {
 			goto end0;
 		}
@@ -2052,12 +2053,14 @@ void uwsgi_master_manage_emperor() {
                 uwsgi_log_verbose("received message %d from emperor\n", byte);
                 // remove me
                 if (byte == 0) {
+			uwsgi_hooks_run(uwsgi.hook_emperor_stop, "emperor-stop", 0);
                         close(uwsgi.emperor_fd);
                         if (!uwsgi.status.brutally_reloading)
                                 kill_them_all(0);
                 }
                 // reload me
                 else if (byte == 1) {
+			uwsgi_hooks_run(uwsgi.hook_emperor_reload, "emperor-reload", 0);
                         // un-lazy the stack to trigger a real reload
                         uwsgi.lazy = 0;
                         uwsgi_block_signal(SIGHUP);
@@ -2067,6 +2070,7 @@ void uwsgi_master_manage_emperor() {
         }
         else {
                 uwsgi_log("lost connection with my emperor !!!\n");
+		uwsgi_hooks_run(uwsgi.hook_emperor_lost, "emperor-lost", 0);
                 close(uwsgi.emperor_fd);
                 if (!uwsgi.status.brutally_reloading)
                         kill_them_all(0);
