@@ -15,7 +15,7 @@ int uwsgi_waitfd_event(int fd, int timeout, int event) {
 	struct pollfd upoll;
 
 	if (!timeout)
-		timeout = uwsgi.shared->options[UWSGI_OPTION_SOCKET_TIMEOUT];
+		timeout = uwsgi.socket_timeout;
 
 	timeout = timeout * 1000;
 	if (timeout < 0)
@@ -586,11 +586,13 @@ int *uwsgi_attach_fd(int fd, int *count_ptr, char *code, size_t code_len) {
 	len = recvmsg(fd, &msg, 0);
 	if (len <= 0) {
 		uwsgi_error("recvmsg()");
+		free(msg_control);
 		return NULL;
 	}
 
 	if (code && code_len > 0) {
 		if (uwsgi_strncmp(id, code_len, code, code_len)) {
+			free(msg_control);
 			return NULL;
 		}
 
@@ -606,15 +608,19 @@ int *uwsgi_attach_fd(int fd, int *count_ptr, char *code, size_t code_len) {
 	}
 
 	cmsg = CMSG_FIRSTHDR(&msg);
-	if (!cmsg)
+	if (!cmsg) {
+		free(msg_control);
 		return NULL;
+	}
 
 	if (cmsg->cmsg_level != SOL_SOCKET || cmsg->cmsg_type != SCM_RIGHTS) {
+		free(msg_control);
 		return NULL;
 	}
 
 	if ((size_t) (cmsg->cmsg_len - ((char *) CMSG_DATA(cmsg) - (char *) cmsg)) > (size_t) (sizeof(int) * (count + 1))) {
 		uwsgi_log("not enough space for sockets data, consider increasing it\n");
+		free(msg_control);
 		return NULL;
 	}
 
@@ -1216,3 +1222,204 @@ void uwsgi_remap_fd(int fd, char *filename) {
         }
 }
 
+int uwsgi_is_connected(int fd) {
+	int soopt;
+        socklen_t solen = sizeof(int);
+
+        if (getsockopt(fd, SOL_SOCKET, SO_ERROR, (void *) (&soopt), &solen) < 0) {
+		return 0;
+        }
+        /* is something bad ? */
+        if (soopt) return 0;
+	return 1;
+}
+
+
+int uwsgi_pass_cred(int fd, char *code, size_t code_len) {
+#ifdef SCM_CREDENTIALS
+	struct msghdr cr_msg;
+        struct cmsghdr *cmsg;
+        struct iovec cr_iov;
+        void *cr_msg_control = uwsgi_calloc(CMSG_SPACE(sizeof(struct ucred)));
+
+        cr_iov.iov_base = code;
+        cr_iov.iov_len = code_len;
+
+        cr_msg.msg_name = NULL;
+        cr_msg.msg_namelen = 0;
+
+        cr_msg.msg_iov = &cr_iov;
+        cr_msg.msg_iovlen = 1;
+
+        cr_msg.msg_flags = 0;
+        cr_msg.msg_control = cr_msg_control;
+        cr_msg.msg_controllen = CMSG_SPACE(sizeof(struct ucred));
+
+        cmsg = CMSG_FIRSTHDR(&cr_msg);
+        cmsg->cmsg_len = CMSG_LEN(sizeof(struct ucred));
+        cmsg->cmsg_level = SOL_SOCKET;
+        cmsg->cmsg_type = SCM_CREDENTIALS;
+
+        struct ucred *u = (struct ucred*) CMSG_DATA(cmsg);
+	u->pid = getpid();	
+	u->uid = getuid();	
+	u->gid = getgid();	
+
+        if (sendmsg(fd, &cr_msg, 0) < 0) {
+                uwsgi_error("uwsgi_pass_cred()/sendmsg()");
+        	free(cr_msg_control);
+		return -1;
+        }
+
+        free(cr_msg_control);
+	return 0;
+#else
+	return -1;
+#endif
+}
+
+int uwsgi_pass_cred2(int fd, char *code, size_t code_len, struct sockaddr *addr, size_t addr_len) {
+#ifdef SCM_CREDENTIALS
+        struct msghdr cr_msg;
+        struct cmsghdr *cmsg;
+        struct iovec cr_iov;
+        void *cr_msg_control = uwsgi_calloc(CMSG_SPACE(sizeof(struct ucred)));
+
+        cr_iov.iov_base = code;
+        cr_iov.iov_len = code_len;
+
+        cr_msg.msg_name = addr;
+        cr_msg.msg_namelen = addr_len;
+
+        cr_msg.msg_iov = &cr_iov;
+        cr_msg.msg_iovlen = 1;
+
+        cr_msg.msg_flags = 0;
+        cr_msg.msg_control = cr_msg_control;
+        cr_msg.msg_controllen = CMSG_SPACE(sizeof(struct ucred));
+
+        cmsg = CMSG_FIRSTHDR(&cr_msg);
+        cmsg->cmsg_len = CMSG_LEN(sizeof(struct ucred));
+        cmsg->cmsg_level = SOL_SOCKET;
+        cmsg->cmsg_type = SCM_CREDENTIALS;
+
+        struct ucred *u = (struct ucred*) CMSG_DATA(cmsg);
+        u->pid = getpid();
+        u->uid = getuid();
+        u->gid = getgid();
+
+        if (sendmsg(fd, &cr_msg, 0) < 0) {
+                uwsgi_error("uwsgi_pass_cred2()/sendmsg()");
+                free(cr_msg_control);
+                return -1;
+        }
+
+        free(cr_msg_control);
+        return 0;
+#else
+        return -1;
+#endif
+}
+
+
+int uwsgi_recv_cred(int fd, char *code, size_t code_len, pid_t *pid, uid_t *uid, gid_t *gid) {
+#ifdef SCM_CREDENTIALS
+        struct iovec iov;
+	int ret = -1;
+
+        void *msg_control = uwsgi_calloc(CMSG_SPACE(sizeof(struct ucred)));
+
+        iov.iov_base = uwsgi_malloc(code_len);
+        iov.iov_len = code_len;
+
+	struct msghdr msg;
+        memset(&msg, 0, sizeof(msg));
+
+        msg.msg_name = NULL;
+        msg.msg_namelen = 0;
+
+        msg.msg_iov = &iov;
+        msg.msg_iovlen = 1;
+
+        msg.msg_control = msg_control;
+        msg.msg_controllen = CMSG_SPACE(sizeof(struct ucred));
+
+        ssize_t len = recvmsg(fd, &msg, 0);
+	if (len <= 0) {
+		uwsgi_error("uwsgi_recv_cred()/recvmsg()");
+		goto clear;
+	}
+
+	struct cmsghdr *cmsg = CMSG_FIRSTHDR(&msg);
+	if (!cmsg) goto clear;
+
+	if (cmsg->cmsg_level != SOL_SOCKET || cmsg->cmsg_type != SCM_CREDENTIALS) {
+		goto clear;
+        }
+
+	if (uwsgi_strncmp(code, code_len, iov.iov_base, iov.iov_len)) goto clear;
+
+	struct ucred *u = (struct ucred *) CMSG_DATA(cmsg);
+	*pid = u->pid;
+	*uid = u->uid;
+	*gid = u->gid;
+	ret = 0;
+
+clear:
+	free(msg_control);
+	free(iov.iov_base);
+	return ret;
+#else
+	return -1;
+#endif
+}
+
+ssize_t uwsgi_recv_cred2(int fd, char *buf, size_t buf_len, pid_t *pid, uid_t *uid, gid_t *gid) {
+#ifdef SCM_CREDENTIALS
+        struct iovec iov;
+        ssize_t ret = -1;
+
+        void *msg_control = uwsgi_calloc(CMSG_SPACE(sizeof(struct ucred)));
+
+        iov.iov_base = buf;
+        iov.iov_len = buf_len;
+
+        struct msghdr msg;
+        memset(&msg, 0, sizeof(msg));
+
+        msg.msg_name = NULL;
+        msg.msg_namelen = 0;
+
+        msg.msg_iov = &iov;
+        msg.msg_iovlen = 1;
+
+        msg.msg_control = msg_control;
+        msg.msg_controllen = CMSG_SPACE(sizeof(struct ucred));
+
+        ssize_t len = recvmsg(fd, &msg, 0);
+        if (len <= 0) {
+                uwsgi_error("uwsgi_recv_cred2()/recvmsg()");
+                goto clear;
+        }
+
+        struct cmsghdr *cmsg = CMSG_FIRSTHDR(&msg);
+        if (!cmsg) goto clear;
+
+        if (cmsg->cmsg_level != SOL_SOCKET || cmsg->cmsg_type != SCM_CREDENTIALS) {
+                goto clear;
+        }
+
+        struct ucred *u = (struct ucred *) CMSG_DATA(cmsg);
+        *pid = u->pid;
+        *uid = u->uid;
+        *gid = u->gid;
+        ret = len;
+
+clear:
+        free(msg_control);
+        return ret;
+#else
+        return -1;
+#endif
+}
+
