@@ -101,7 +101,7 @@ int uwsgi_calc_cheaper(void) {
 
 	int i;
 	static time_t last_check = 0;
-	int check_interval = uwsgi.shared->options[UWSGI_OPTION_MASTER_INTERVAL];
+	int check_interval = uwsgi.master_interval;
 
 	if (!last_check)
 		last_check = uwsgi_now();
@@ -186,6 +186,8 @@ int uwsgi_calc_cheaper(void) {
 			uwsgi_log("worker %d should die...\n", oldest_worker);
 #endif
 			uwsgi.workers[oldest_worker].cheaped = 1;
+			uwsgi.workers[oldest_worker].rss_size = 0;
+			uwsgi.workers[oldest_worker].vsz_size = 0;
 			uwsgi.workers[oldest_worker].manage_next_request = 0;
 			uwsgi_curse(oldest_worker, SIGWINCH);
 		}
@@ -330,7 +332,7 @@ int uwsgi_cheaper_algo_backlog(int can_spawn) {
 
 	int i;
 #ifdef __linux__
-	int backlog = uwsgi.shared->options[UWSGI_OPTION_BACKLOG_STATUS];
+	int backlog = uwsgi.shared->backlog;
 #else
 	int backlog = 0;
 #endif
@@ -383,14 +385,14 @@ void uwsgi_reload(char **argv) {
 		// call master cleanup hooks
 		uwsgi_master_cleanup_hooks();
 
-		// call atexit user exec
-		uwsgi_exec_atexit();
-
 		if (uwsgi.exit_on_reload) {
 			uwsgi_log("uWSGI: GAME OVER (insert coin)\n");
 			exit(0);
 		}
 
+		// call atexit user exec
+		uwsgi_exec_atexit();
+
 		uwsgi_log("binary reloading uWSGI...\n");
 	}
 	else {
@@ -584,6 +586,19 @@ void uwsgi_fixup_fds(int wid, int muleid, struct uwsgi_gateway *ug) {
 		}
 
 		if (uwsgi.master_fifo_fd > -1) close(uwsgi.master_fifo_fd);
+
+#ifdef __linux__
+		for(i=0;i<uwsgi.setns_fds_count;i++) {
+			close(uwsgi.setns_fds[i]);
+		}
+#endif
+
+		// fd alarms
+		struct uwsgi_alarm_fd *uafd = uwsgi.alarm_fds;
+        	while(uafd) {
+			close(uafd->fd);
+                	uafd = uafd->next;
+        	}
 	}
 
 
@@ -713,9 +728,9 @@ struct uwsgi_stats *uwsgi_master_generate_stats() {
 		goto end;
 
 #ifdef __linux__
-	if (uwsgi_stats_keylong_comma(us, "listen_queue", (unsigned long long) uwsgi.shared->options[UWSGI_OPTION_BACKLOG_STATUS]))
+	if (uwsgi_stats_keylong_comma(us, "listen_queue", (unsigned long long) uwsgi.shared->backlog))
 		goto end;
-	if (uwsgi_stats_keylong_comma(us, "listen_queue_errors", (unsigned long long) uwsgi.shared->options[UWSGI_OPTION_BACKLOG_ERRORS]))
+	if (uwsgi_stats_keylong_comma(us, "listen_queue_errors", (unsigned long long) uwsgi.shared->backlog_errors))
 		goto end;
 #endif
 
@@ -957,6 +972,9 @@ struct uwsgi_stats *uwsgi_master_generate_stats() {
 		if (uwsgi_stats_keylong_comma(us, "queue", (unsigned long long) uwsgi_sock->queue))
 			goto end;
 
+		if (uwsgi_stats_keylong_comma(us, "max_queue", (unsigned long long) uwsgi_sock->max_queue))
+			goto end;
+
 		if (uwsgi_stats_keylong_comma(us, "shared", (unsigned long long) uwsgi_sock->shared))
 			goto end;
 
@@ -1080,7 +1098,7 @@ struct uwsgi_stats *uwsgi_master_generate_stats() {
 			if (uwsgi_stats_keylong_comma(us, "exceptions", ua->exceptions))
 				goto end;
 
-			if (ua->chdir) {
+			if (*ua->chdir) {
 				if (uwsgi_stats_keyval(us, "chdir", ua->chdir))
 					goto end;
 			}
@@ -1453,7 +1471,7 @@ void uwsgi_register_cheaper_algo(char *name, int (*func) (int)) {
 
 void trigger_harakiri(int i) {
 	int j;
-	uwsgi_log("*** HARAKIRI ON WORKER %d (pid: %d, try: %d) ***\n", i, uwsgi.workers[i].pid, uwsgi.workers[i].pending_harakiri + 1);
+	uwsgi_log_verbose("*** HARAKIRI ON WORKER %d (pid: %d, try: %d) ***\n", i, uwsgi.workers[i].pid, uwsgi.workers[i].pending_harakiri + 1);
 	if (uwsgi.harakiri_verbose) {
 #ifdef __linux__
 		int proc_file;
@@ -1501,15 +1519,12 @@ void trigger_harakiri(int i) {
 			}
 		}
 
-		kill(uwsgi.workers[i].pid, SIGUSR2);
-		// allow SIGUSR2 to be delivered
-		sleep(1);
+		uwsgi_dump_worker(i, "HARAKIRI");
 		kill(uwsgi.workers[i].pid, SIGKILL);
 		if (!uwsgi.workers[i].pending_harakiri)
 			uwsgi.workers[i].harakiri_count++;
 		uwsgi.workers[i].pending_harakiri++;
 	}
-	// to avoid races
 
 }
 
@@ -1649,6 +1664,8 @@ void uwsgi_go_cheap() {
 	uwsgi.status.is_cheap = 1;
                 for (i = 1; i <= uwsgi.numproc; i++) {
                         uwsgi.workers[i].cheaped = 1;
+			uwsgi.workers[i].rss_size = 0;
+        		uwsgi.workers[i].vsz_size = 0;
                         if (uwsgi.workers[i].pid == 0)
                                 continue;
 			uwsgi_log("killing worker %d (pid: %d)\n", i, (int) uwsgi.workers[i].pid);
@@ -1661,3 +1678,154 @@ void uwsgi_go_cheap() {
                 uwsgi_add_sockets_to_queue(uwsgi.master_queue, -1);
                 uwsgi_log("cheap mode enabled: waiting for socket connection...\n");
 }
+
+#ifdef __linux__
+void uwsgi_setns_preopen() {
+	struct dirent *de;
+        DIR *ns = opendir("/proc/self/ns");
+        if (!ns) {
+                uwsgi_error("uwsgi_setns_preopen()/opendir()");
+		exit(1);
+        }
+        while ((de = readdir(ns)) != NULL) {
+                if (strlen(de->d_name) > 0 && de->d_name[0] == '.') continue;
+                struct uwsgi_string_list *usl = NULL;
+                int found = 0;
+                uwsgi_foreach(usl, uwsgi.setns_socket_skip) {
+                        if (!strcmp(de->d_name, usl->value)) {
+                                found = 1;
+                                break;
+                        }
+                }
+                if (found) continue;
+                char *filename = uwsgi_concat2("/proc/self/ns/", de->d_name);
+                uwsgi.setns_fds[uwsgi.setns_fds_count] = open(filename, O_RDONLY);
+                if (uwsgi.setns_fds[uwsgi.setns_fds_count] < 0) {
+                        uwsgi_error_open(filename);
+                        free(filename);
+			exit(1);
+                }
+                free(filename);
+                uwsgi.setns_fds_count++;
+        }
+	closedir(ns);
+}
+void uwsgi_master_manage_setns(int fd) {
+
+        struct sockaddr_un snsun;
+        socklen_t snsun_len = sizeof(struct sockaddr_un);
+
+        int setns_client = accept(fd, (struct sockaddr *) &snsun, &snsun_len);
+        if (setns_client < 0) {
+                uwsgi_error("uwsgi_master_manage_setns()/accept()");
+                return;
+        }
+
+	int i;
+	int tmp_fds[64];
+	int *fds = tmp_fds;
+        int num_fds = 0;
+
+	struct msghdr sn_msg;
+        void *sn_msg_control;
+        struct iovec sn_iov[2];
+        struct cmsghdr *cmsg;
+	DIR *ns = NULL;
+
+	if (uwsgi.setns_fds_count) {
+		fds = uwsgi.setns_fds;
+		num_fds = uwsgi.setns_fds_count;
+		goto send;
+	}
+
+	struct dirent *de;
+	ns = opendir("/proc/self/ns");
+	if (!ns) {
+		close(setns_client);
+		uwsgi_error("uwsgi_master_manage_setns()/opendir()");
+		return;
+	}
+	while ((de = readdir(ns)) != NULL) {
+		if (strlen(de->d_name) > 0 && de->d_name[0] == '.') continue;
+		struct uwsgi_string_list *usl = NULL;
+		int found = 0;
+		uwsgi_foreach(usl, uwsgi.setns_socket_skip) {
+			if (!strcmp(de->d_name, usl->value)) {
+				found = 1;
+				break;
+			}
+		}
+		if (found) continue;
+		char *filename = uwsgi_concat2("/proc/self/ns/", de->d_name);
+		fds[num_fds] = open(filename, O_RDONLY);
+		if (fds[num_fds] < 0) {
+			uwsgi_error_open(filename);
+			free(filename);
+			goto clear;
+		}
+		free(filename);
+		num_fds++;
+	}
+
+send:
+
+        sn_msg_control = uwsgi_malloc(CMSG_SPACE(sizeof(int) * num_fds));
+
+        sn_iov[0].iov_base = "uwsgi-setns";
+        sn_iov[0].iov_len = 11;
+        sn_iov[1].iov_base = &num_fds;
+        sn_iov[1].iov_len = sizeof(int);
+
+        sn_msg.msg_name = NULL;
+        sn_msg.msg_namelen = 0;
+
+        sn_msg.msg_iov = sn_iov;
+        sn_msg.msg_iovlen = 2;
+
+        sn_msg.msg_flags = 0;
+        sn_msg.msg_control = sn_msg_control;
+        sn_msg.msg_controllen = CMSG_SPACE(sizeof(int) * num_fds);
+
+        cmsg = CMSG_FIRSTHDR(&sn_msg);
+        cmsg->cmsg_len = CMSG_LEN(sizeof(int) * num_fds);
+        cmsg->cmsg_level = SOL_SOCKET;
+        cmsg->cmsg_type = SCM_RIGHTS;
+
+        int *sn_fd_ptr = (int *) CMSG_DATA(cmsg);
+	for(i=0;i<num_fds;i++) {
+		sn_fd_ptr[i] = fds[i];
+	}
+
+        if (sendmsg(setns_client, &sn_msg, 0) < 0) {
+                uwsgi_error("uwsgi_master_manage_setns()/sendmsg()");
+        }
+
+        free(sn_msg_control);
+
+clear:
+	close(setns_client);
+	if (ns) {
+		closedir(ns);
+		for(i=0;i<num_fds;i++) {
+			close(fds[i]);
+		}
+	}
+}
+
+#endif
+
+/*
+	this is racey, but the worst thing would be printing garbage in the logs...
+*/
+void uwsgi_dump_worker(int wid, char *msg) {
+	int i;
+	uwsgi_log_verbose("%s !!! worker %d status !!!\n", msg, wid);
+	for(i=0;i<uwsgi.cores;i++) {
+		struct uwsgi_core *uc = &uwsgi.workers[wid].cores[i];
+		struct wsgi_request *wsgi_req = &uc->req;
+		if (uc->in_request) {
+			uwsgi_log_verbose("%s [core %d] %.*s - %.*s %.*s since %llu\n", msg, i, wsgi_req->remote_addr_len, wsgi_req->remote_addr, wsgi_req->method_len, wsgi_req->method, wsgi_req->uri_len, wsgi_req->uri, (unsigned long long) (wsgi_req->start_of_request/(1000*1000)));
+		}
+	}
+	uwsgi_log_verbose("%s !!! end of worker %d status !!!\n",msg, wid);
+}
