@@ -44,6 +44,8 @@ XS(XS_wait_fd_read) {
 		croak("unable to add fd %d to the event queue", fd);
 	}
 
+	wsgi_req->async_force_again = 1;
+
 	XSRETURN_UNDEF;
 }
 
@@ -243,6 +245,13 @@ XS(XS_register_signal) {
 	
 }
 
+XS(XS_spooler) {
+	dXSARGS;
+	psgi_check_args(1);
+	uperl.spooler = (CV *) newRV_inc(ST(0));
+	XSRETURN_YES;
+}
+
 XS(XS_register_rpc) {
         dXSARGS;
 
@@ -319,6 +328,14 @@ XS(XS_async_connect) {
 	XSRETURN(1);
 }
 
+XS(XS_ready_fd) {
+	dXSARGS;
+        psgi_check_args(0);
+	struct wsgi_request *wsgi_req = current_wsgi_req();	
+	ST(0) = newSViv(uwsgi_ready_fd(wsgi_req));
+	XSRETURN(1);
+}
+
 XS(XS_call) {
 
 	dXSARGS;
@@ -463,16 +480,24 @@ XS(XS_websocket_handshake) {
         char *origin = NULL;
 	STRLEN origin_len = 0;
 
-	psgi_check_args(1);
-	
-	key = SvPV(ST(0), key_len);
+	char *proto = NULL;
+	STRLEN proto_len = 0;
 
-	if (items > 1) {
-		origin = SvPV(ST(0), origin_len);
+	psgi_check_args(0);
+	
+	if (items > 0) {
+		key = SvPV(ST(0), key_len);
+		if (items > 1) {
+			origin = SvPV(ST(1), origin_len);
+			if (items > 2) {
+				proto = SvPV(ST(2), proto_len);
+			}
+		}
+		
 	}
         struct wsgi_request *wsgi_req = current_wsgi_req();
 
-        if (uwsgi_websocket_handshake(wsgi_req, key, key_len, origin, origin_len)) {
+        if (uwsgi_websocket_handshake(wsgi_req, key, key_len, origin, origin_len, proto, proto_len)) {
                 croak("unable to complete websocket handshake");
 	}
 
@@ -886,6 +911,7 @@ void init_perl_embedded_module() {
 	psgi_xs(wait_fd_read);
 	psgi_xs(wait_fd_write);
 	psgi_xs(async_sleep);
+	psgi_xs(ready_fd);
 	psgi_xs(log);
 	psgi_xs(async_connect);
 	psgi_xs(suspend);
@@ -929,4 +955,6 @@ void init_perl_embedded_module() {
 	psgi_xs(sharedarea_readfast);
 	psgi_xs(sharedarea_write);
 	psgi_xs(sharedarea_wait);
+
+	psgi_xs(spooler);
 }
