@@ -174,7 +174,18 @@ char *uwsgi_encode_pydict(PyObject * pydict, uint16_t * size) {
 
 static PyObject *py_uwsgi_listen_queue(PyObject * self, PyObject * args) {
 
-	return PyInt_FromLong(uwsgi.shared->options[UWSGI_OPTION_BACKLOG_STATUS]);
+	int id = 0;
+
+	if (!PyArg_ParseTuple(args, "|i:listen_queue", &id)) {
+                return NULL;
+        }
+
+	struct uwsgi_socket *uwsgi_sock = uwsgi_get_socket_by_num(id);
+	if (!uwsgi_sock) {
+		return PyErr_Format(PyExc_ValueError, "unable to find socket %d", id);
+	}
+
+	return PyInt_FromLong(uwsgi_sock->queue);
 }
 
 static PyObject *py_uwsgi_close(PyObject * self, PyObject * args) {
@@ -187,7 +198,6 @@ static PyObject *py_uwsgi_close(PyObject * self, PyObject * args) {
 
 	close(fd);
 
-
 	Py_INCREF(Py_None);
 	return Py_None;
 
@@ -604,59 +614,6 @@ PyObject *py_uwsgi_set_logvar(PyObject * self, PyObject * args) {
         return Py_None;
 }
 
-PyObject *py_uwsgi_recv_block(PyObject * self, PyObject * args) {
-
-	char buf[4096];
-	char *bufptr;
-	ssize_t rlen = 0, len;
-	int fd, size, remains, ret, timeout = -1;
-
-
-	if (!PyArg_ParseTuple(args, "ii|i:recv_block", &fd, &size, &timeout)) {
-		return NULL;
-	}
-
-	if (fd < 0)
-		goto clear;
-
-	UWSGI_RELEASE_GIL
-		// security check
-		if (size > 4096)
-		size = 4096;
-
-	remains = size;
-
-	bufptr = buf;
-	while (remains > 0) {
-		uwsgi_log("%d %d %d\n", remains, size, timeout);
-		ret = uwsgi_waitfd(fd, timeout);
-		if (ret > 0) {
-			len = read(fd, bufptr, UMIN(remains, size));
-			if (len > 0) {
-				bufptr += len;
-				rlen += len;
-				remains -= len;
-			}
-			else {
-				break;
-			}
-		}
-		else {
-			uwsgi_log("error waiting for block data\n");
-			break;
-		}
-	}
-
-	UWSGI_GET_GIL if (rlen == size) {
-		return PyString_FromStringAndSize(buf, rlen);
-	}
-
-      clear:
-
-	Py_INCREF(Py_None);
-	return Py_None;
-}
-
 PyObject *py_uwsgi_recv(PyObject * self, PyObject * args) {
 
 	int fd, max_size = 4096;
@@ -685,28 +642,19 @@ PyObject *py_uwsgi_recv(PyObject * self, PyObject * args) {
 
 PyObject *py_uwsgi_is_connected(PyObject * self, PyObject * args) {
 
-	int fd, soopt;
-	socklen_t solen = sizeof(int);
+	int fd = -1;
 
 	if (!PyArg_ParseTuple(args, "i:is_connected", &fd)) {
 		return NULL;
 	}
 
-	if (getsockopt(fd, SOL_SOCKET, SO_ERROR, (void *) (&soopt), &solen) < 0) {
-		uwsgi_error("getsockopt()");
-		goto clear;
+	if (uwsgi_is_connected(fd)) {
+		Py_INCREF(Py_True);
+		return Py_True;
 	}
-	/* is something bad ? */
-	if (soopt)
-		goto clear;
 
-	Py_INCREF(Py_True);
-	return Py_True;
-
-      clear:
-
-	Py_INCREF(Py_None);
-	return Py_None;
+	Py_INCREF(Py_False);
+	return Py_False;
 }
 
 
@@ -1028,14 +976,17 @@ PyObject *py_uwsgi_websocket_handshake(PyObject * self, PyObject * args) {
         char *origin = NULL;
         Py_ssize_t origin_len = 0;
 
-        if (!PyArg_ParseTuple(args, "s#|s#:websocket_handshake", &key, &key_len, &origin, &origin_len)) {
+        char *proto = NULL;
+        Py_ssize_t proto_len = 0;
+
+        if (!PyArg_ParseTuple(args, "|s#s#s#:websocket_handshake", &key, &key_len, &origin, &origin_len, &proto, &proto_len)) {
                 return NULL;
         }
 
 	struct wsgi_request *wsgi_req = py_current_wsgi_req();
 
 	UWSGI_RELEASE_GIL
-	int ret = uwsgi_websocket_handshake(wsgi_req, key, key_len, origin, origin_len);
+	int ret = uwsgi_websocket_handshake(wsgi_req, key, key_len, origin, origin_len, proto, proto_len);
 	UWSGI_GET_GIL
 
 	if (ret) {
@@ -1991,46 +1942,6 @@ PyObject *py_uwsgi_spooler_pid(PyObject * self, PyObject * args) {
 }
 
 
-PyObject *py_uwsgi_get_option(PyObject * self, PyObject * args) {
-	int opt_id;
-
-	if (!PyArg_ParseTuple(args, "i:get_option", &opt_id)) {
-		return NULL;
-	}
-
-	return PyInt_FromLong(uwsgi.shared->options[(uint8_t) opt_id]);
-}
-
-PyObject *py_uwsgi_set_option(PyObject * self, PyObject * args) {
-	int opt_id;
-	int value;
-
-	if (!PyArg_ParseTuple(args, "ii:set_option", &opt_id, &value)) {
-		return NULL;
-	}
-
-	uwsgi.shared->options[(uint8_t) opt_id] = (uint32_t) value;
-	return PyInt_FromLong(value);
-}
-
-PyObject *py_uwsgi_has_hook(PyObject * self, PyObject * args) {
-	int modifier1;
-
-	if (!PyArg_ParseTuple(args, "i:has_hook", &modifier1)) {
-		return NULL;
-	}
-
-	/*
-	   if (uwsgi.shared->hooks[modifier1] != unconfigured_hook) {
-	   Py_INCREF(Py_True);
-	   return Py_True;
-	   }
-	 */
-
-	Py_INCREF(Py_None);
-	return Py_None;
-}
-
 PyObject *py_uwsgi_connect(PyObject * self, PyObject * args) {
 
 	char *socket_name = NULL;
@@ -2225,7 +2136,7 @@ PyObject *py_uwsgi_workers(PyObject * self, PyObject * args) {
 			PyDict_SetItemString(apps_dict, "exceptions", zero);
 			Py_DECREF(zero);
 
-			if (ua->chdir) {
+			if (*ua->chdir) {
 				zero = PyString_FromString(ua->chdir);
 			}
 			else {
@@ -2281,21 +2192,6 @@ PyObject *py_uwsgi_stop(PyObject * self, PyObject * args) {
         return Py_True;
 }
 
-
-	/* blocking hint */
-PyObject *py_uwsgi_set_blocking(PyObject * self, PyObject * args) {
-
-	if (uwsgi.master_process) {
-		Py_INCREF(Py_True);
-		return Py_True;
-	}
-
-
-	Py_INCREF(Py_None);
-	return Py_None;
-}
-
-
 PyObject *py_uwsgi_request_id(PyObject * self, PyObject * args) {
 	return PyLong_FromUnsignedLongLong(uwsgi.workers[uwsgi.mywid].requests);
 }
@@ -2497,11 +2393,6 @@ static PyMethodDef uwsgi_advanced_methods[] = {
 	{"workers", py_uwsgi_workers, METH_VARARGS, ""},
 	{"masterpid", py_uwsgi_masterpid, METH_VARARGS, ""},
 	{"total_requests", py_uwsgi_total_requests, METH_VARARGS, ""},
-	{"getoption", py_uwsgi_get_option, METH_VARARGS, ""},
-	{"get_option", py_uwsgi_get_option, METH_VARARGS, ""},
-	{"setoption", py_uwsgi_set_option, METH_VARARGS, ""},
-	{"set_option", py_uwsgi_set_option, METH_VARARGS, ""},
-	{"sorry_i_need_to_block", py_uwsgi_set_blocking, METH_VARARGS, ""},
 	{"request_id", py_uwsgi_request_id, METH_VARARGS, ""},
 	{"worker_id", py_uwsgi_worker_id, METH_VARARGS, ""},
 	{"mule_id", py_uwsgi_mule_id, METH_VARARGS, ""},
@@ -2542,7 +2433,6 @@ static PyMethodDef uwsgi_advanced_methods[] = {
 	{"offload", py_uwsgi_offload, METH_VARARGS, ""},
 	{"set_warning_message", py_uwsgi_warning, METH_VARARGS, ""},
 	{"mem", py_uwsgi_mem, METH_VARARGS, ""},
-	{"has_hook", py_uwsgi_has_hook, METH_VARARGS, ""},
 	{"logsize", py_uwsgi_logsize, METH_VARARGS, ""},
 #ifdef UWSGI_SSL
 	{"i_am_the_lord", py_uwsgi_i_am_the_lord, METH_VARARGS, ""},
@@ -2562,7 +2452,6 @@ static PyMethodDef uwsgi_advanced_methods[] = {
 	{"is_connected", py_uwsgi_is_connected, METH_VARARGS, ""},
 	{"send", py_uwsgi_send, METH_VARARGS, ""},
 	{"recv", py_uwsgi_recv, METH_VARARGS, ""},
-	{"recv_block", py_uwsgi_recv_block, METH_VARARGS, ""},
 	{"close", py_uwsgi_close, METH_VARARGS, ""},
 	{"i_am_the_spooler", py_uwsgi_i_am_the_spooler, METH_VARARGS, ""},
 
